#include <string.h>


#include "dialogs.h"
#include "ferret.h"
#include "search.h"
#include "properties.h"
#include "preferences.h"
#include "stringtokenizer.h"
#include "webupdate.h"


GtkWidget	*properties_window;
GList		*engines_to_add;
GList		*engines_to_remove;
gint		active_engine_count = 0;
/*gint		done_web_update_this_session = FALSE */
GtkWidget	*web_update;
gint		property_box_changed = FALSE;


/*	displays the ferret about dialog box, drawing its'
	information from the ".netferretrc" properties file
*/
void about_dialog (AppletWidget *applet, gpointer data)
{
	GtkWidget *about;
	gchar *authtxt = get_property ("about_auth");
	TokenizerPtr tok = new_tokenizer(authtxt, ",");
	gint len = count_tokens(tok);
	gchar *contribtxt = get_property ("about_contrib");
	TokenizerPtr tok2 = new_tokenizer(contribtxt, ",");
	gint len2 = count_tokens(tok2);
	const gchar *authors[len+len2+1];
	int i=0;
	gchar *tmp, *tmpcpy;
	
	//printf ("author: %s\n", authtxt);
	
	while (has_more_tokens(tok))
	{
		tmp = next_token(tok);
		tmpcpy = malloc (strlen(tmp)+1);
		strcpy (tmpcpy, tmp);
		authors[i++]=tmpcpy;
#ifdef HDEBUG
		h_free (tmp, "tmp", "about_dialog", 44);
#else
		g_free (tmp);
#endif
	}
	while (has_more_tokens(tok2))
	{
		tmp = next_token(tok2);
		tmpcpy = malloc (strlen(tmp)+1);
		strcpy (tmpcpy, tmp);
		authors[i++]=tmpcpy;
#ifdef HDEBUG
		h_free (tmp, "tmp", "about_dialog", 52);
#else
		g_free (tmp);
#endif
	}
	authors[i] = NULL;
		
	about = gnome_about_new (_(get_property("about_title")),
							 FERRET_VERSION,
							 _(get_property("about_copyright")),
							 authors,
							 _(get_property("about_text")),
							 get_property("logo_pixmap"));
	gtk_widget_show (about);
	
	free_tokenizer (tok);
	free_tokenizer (tok2);
}

/*	call before using properties_dialog().
*/
void init_properties_dialog()
{
	properties_window = NULL;
	engines_to_add = NULL;
	engines_to_remove = NULL;
	web_update = NULL;
	active_engine_count = 0;
	property_box_changed = FALSE;
}

/*	call after properties_dialog() is destroyed.
*/
void dispose_properties_dialog()
{
	properties_window = NULL;
	engines_to_add = NULL;
	engines_to_remove = NULL;
	web_update = NULL;
	active_engine_count = 0;
	property_box_changed = FALSE;
}

/*	window destroy callback
*/
void _properties_window_destroy_cb( GtkWidget *widget, gpointer data )
{
	dispose_properties_dialog();
}

/*	apply changes made to properties window.
*/
void _properties_window_apply_cb (GnomePropertyBox *pbox, gint page, gpointer data)
{	
	gint len, i;
	gchar *eng;
	GList *leng;
	
	switch (page)
	{
		/* use engines list page */
		case 0:
		{			
			len = g_list_length (engines_to_add);
			for (i=0;i<len;i++)
			{
				leng = g_list_nth (engines_to_add, i);
				eng = (gchar*)leng->data;
				set_use_engine (eng, TRUE);
			}

			len = g_list_length (engines_to_remove);
			for (i=0;i<len;i++)
			{
				leng = g_list_nth (engines_to_remove, i);
				eng = (gchar*)leng->data;
				set_use_engine (eng, FALSE);
			}

			save_prefs();
			load_prefs();
			init_engine_popup();

			g_list_free (engines_to_add);
			g_list_free (engines_to_remove);

			engines_to_add = NULL;
			engines_to_remove = NULL;
			property_box_changed = FALSE;
		}
	}
}



gchar *_g_list_text_find (GList *list, gchar* text)
{
	gint len = g_list_length(list);
	gint i;
	GList *leng;
	gchar *entry;
	
	for (i=0;i<len;i++)
	{
		leng = g_list_nth (list, i);
		entry = (gchar*)leng->data;
		if (strcmp (entry, text) == 0) return entry;
	}
	return NULL;
}



/*	callback when engine checkbox item is clicked.
*/
void _engine_property_clicked_cb (GtkWidget *tb, gpointer data)
{
	gchar *eng = GTK_LABEL(GTK_BIN (tb)->child)->label;
	gchar *leng;
	GtkWidget *alert;
		
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tb)))
	{
		leng = _g_list_text_find (engines_to_remove, eng);
		if (leng != NULL)
		{
			engines_to_remove = g_list_remove_link (
				engines_to_remove, g_list_find(
					engines_to_remove, leng));
		}
		engines_to_add = g_list_append (engines_to_add, eng);
		active_engine_count ++;
	}
	else
	{
		active_engine_count --;
		if (active_engine_count <= 0)
		{
			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tb), TRUE);
			alert = gnome_warning_dialog(get_property ("no_eng_err"));
			return;
		}
		leng = _g_list_text_find (engines_to_add, eng);
		if (leng != NULL)
		{
			engines_to_add = g_list_remove_link (
				engines_to_add, g_list_find(
					engines_to_add, leng));
		}
		engines_to_remove = g_list_append (engines_to_remove, eng);
	}
	
	if (!property_box_changed)
	{
		gnome_property_box_changed(GNOME_PROPERTY_BOX(properties_window));
		property_box_changed = TRUE;
	}
}


/*	called for each engine to build the checkbox items in
	the "properties->engines" tab.
*/
void _properties_window_add_engine (gpointer data, gpointer user_data)
{
	//Engine			*eng = (Engine*)data;
	gchar			*eng = (gchar*)((Engine*)data)->list_name;
	gchar			*engcpy;
	GtkWidget		*toggle;
	EnginesListPtr	engineslist = (EnginesListPtr)user_data;
	gint			is_active;
	
#ifdef HDEBUG
	engcpy = h_alloc (strlen(eng)+1, "engcpy", "_properties_window_add_engine", 213);
#else
	engcpy = g_malloc (strlen(eng)+1);
#endif	
	
	strcpy (engcpy, eng);
	
	toggle = gtk_check_button_new_with_label (engcpy);
	
	//printf ("engine: %s\n", eng);
	
	is_active = user_prefs_contain_engine(eng);
	
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(toggle), is_active);
	
	if (is_active)
		active_engine_count ++;
	
	if (engineslist->current >= 4) engineslist->current = 0;
	
	switch  (engineslist->current)
	{
		case 0: gtk_box_pack_start (GTK_BOX(engineslist->engine_item_list0),
									toggle,
									FALSE, FALSE, DIALOG_SPACER);
		break;
		case 1: gtk_box_pack_start (GTK_BOX(engineslist->engine_item_list1),
									toggle,
									FALSE, FALSE, DIALOG_SPACER);
		break;
		case 2: gtk_box_pack_start (GTK_BOX(engineslist->engine_item_list2),
									toggle,
									FALSE, FALSE, DIALOG_SPACER);
		break;
		case 3: gtk_box_pack_start (GTK_BOX(engineslist->engine_item_list3),
									toggle,
									FALSE, FALSE, DIALOG_SPACER);
		break;
	}
	
	gtk_signal_connect (GTK_OBJECT(toggle), "toggled",
						(GtkSignalFunc)_engine_property_clicked_cb,
						engineslist);
	
	engineslist->current++;
}


/*	The user wants to visit the netferret home page.
	Lanuch the gnome-moz-remote preferred browser with
	the "app_home" property defined in ".netferretrc"
*/
void _app_home_cb ( GtkWidget *widget, gpointer data )
{
	gnome_url_show(get_property ("app_home"));
}


/*	Do the "web update" procedure.
*/
void _web_update_cb ( GtkWidget *widget, gpointer data )
{
/*	if (done_web_update_this_session)
	{
		gtk_widget_set_sensitive (web_update, FALSE);
		return;
	}
*/
	gnome_dialog_close(GNOME_DIALOG(&(GNOME_PROPERTY_BOX(properties_window)->dialog)));
	
	dispose_properties_dialog();
	init_properties_dialog();
	
	install_updated_web_list();
	
	properties_dialog (NULL, NULL);
	
/*	gtk_widget_set_sensitive (web_update, FALSE);
	done_web_update_this_session = TRUE; */
}


/*	initialize the "engines" tab in the "properties" window.
*/
void _init_properties_dialog_components()
{
	GtkWidget	*main_panel;
	GtkWidget	*engines_panel;
	GtkWidget	*button_panel;
	GtkWidget	*app_home;
	EnginesListPtr engineslist;

	//printf ("set title: \"%s\"\n", get_property ("props_dialog_title"));
	gtk_window_set_title(GTK_WINDOW(properties_window), get_property ("props_dialog_title"));

#ifdef HDEBUG
	engineslist = h_alloc (sizeof(EnginesList), "engineslist", "_init_engines_components", 237);
#else
	engineslist = g_malloc (sizeof(EnginesList));
#endif

	/* Build the engines panel */
	//printf ("build engines panel...\n");
	engines_panel = gtk_hbox_new(TRUE, DIALOG_SPACER);
	
	//printf ("build engines lists...\n");
	/* Build a 3 column list of engines as checkbuttons */
	engineslist->current = 0;
	engineslist->engine_item_list0 = gtk_vbox_new(FALSE, DIALOG_SPACER);
	engineslist->engine_item_list1 = gtk_vbox_new(FALSE, DIALOG_SPACER);
	engineslist->engine_item_list2 = gtk_vbox_new(FALSE, DIALOG_SPACER);
	engineslist->engine_item_list3 = gtk_vbox_new(FALSE, DIALOG_SPACER);
	g_list_foreach (get_engines_list(),
					(GFunc)_properties_window_add_engine,
					engineslist);
	gtk_box_pack_start (GTK_BOX(engines_panel),
						engineslist->engine_item_list0,
						FALSE, FALSE, DIALOG_SPACER);
	gtk_box_pack_start (GTK_BOX(engines_panel),
						engineslist->engine_item_list1,
						FALSE, FALSE, DIALOG_SPACER);
	gtk_box_pack_start (GTK_BOX(engines_panel),
						engineslist->engine_item_list2,
						FALSE, FALSE, DIALOG_SPACER);
	gtk_box_pack_start (GTK_BOX(engines_panel),
						engineslist->engine_item_list3,
						FALSE, FALSE, DIALOG_SPACER);
	
	button_panel = gtk_hbox_new(FALSE, DIALOG_SPACER);
		
	web_update = gtk_button_new_with_label (get_property ("web_update_btn"));

	app_home = gtk_button_new_with_label (get_property ("app_home_btn"));
	
	gtk_box_pack_start (GTK_BOX(button_panel),
						web_update,
						FALSE, FALSE, DIALOG_SPACER);
						
	gtk_box_pack_start (GTK_BOX(button_panel),
						app_home,
						FALSE, FALSE, DIALOG_SPACER);
						
	main_panel = gtk_vbox_new(FALSE, DIALOG_SPACER);
	
	gtk_box_pack_start (GTK_BOX(main_panel),
						engines_panel,
						FALSE, FALSE, DIALOG_SPACER);
						
	gtk_box_pack_end (GTK_BOX(main_panel),
					  button_panel,
					  FALSE, FALSE, DIALOG_SPACER);

	gtk_signal_connect (GTK_OBJECT (web_update), "clicked",
						GTK_SIGNAL_FUNC (_web_update_cb), NULL);
	
	gtk_signal_connect (GTK_OBJECT (app_home), "clicked",
						GTK_SIGNAL_FUNC (_app_home_cb), NULL);
	
	gnome_property_box_append_page(GNOME_PROPERTY_BOX (properties_window),
								   main_panel,
								   gtk_label_new ("Engines"));
	
	gtk_signal_connect(GTK_OBJECT(properties_window),
					   "apply",
					   GTK_SIGNAL_FUNC(_properties_window_apply_cb),
					   NULL);
	
	//g_free (engineslist);
	
	gtk_widget_show_all (properties_window);
}


/*	initialize the "properties" window. be sure to call
	"init_properties_dialog" first, and
	"dispose_properties_dialog" when its' destroyed.
*/
void properties_dialog (AppletWidget *applet, gpointer data)
{
	//printf ("new property box\n");
	if (properties_window)
	{
		gdk_window_raise(properties_window->window);
	}
	else
	{
		properties_window = gnome_property_box_new();
		gtk_signal_connect (GTK_OBJECT (properties_window), "delete_event",
							GTK_SIGNAL_FUNC (delete_event), NULL);
		gtk_signal_connect (GTK_OBJECT (properties_window), "destroy",
							GTK_SIGNAL_FUNC (_properties_window_destroy_cb), NULL);
		_init_properties_dialog_components();
	}
}

