
#include "preferences.h"

#include "search.h"
#include "properties.h"
#include "stringtokenizer.h"


Prefs *user_prefs = NULL;


extern GtkWidget	*window;


gint calculate_engine_list_size()
{
	gint len;
	gint i;
	gint ret = 0;
	
	if ((user_prefs == NULL) || (user_prefs->engines == NULL))
	{
		return 0;
	}
	
	len = g_list_length(user_prefs->engines);
	
//	printf ("g_list_length: %d\n", len);

	for (i=0; i<len; i++)
	{
		ret += strlen ((gchar*)g_list_nth(user_prefs->engines, i)->data);
		ret ++;
	}
	
//	printf ("len: %d\n", ret-1);
	return ret;
}

gchar *build_string_from_toggles()
{
	gchar *ret;
	gchar *offs;
	gchar *src;
	gint len;
	gint src_len;
	gint strsize;
	gint i;
	
//	printf ("build_string_from_toggles...\n");

	strsize = calculate_engine_list_size();
	
	if (strsize == 0)
	{
//		printf ("\treturning empty string...\n");
		return "<empty>";
	}
	
#ifdef HDEBUG
	ret = h_alloc (strsize, "ret", "build_string_from_toggles", 60);
#else
	ret = g_malloc (strsize);
#endif

	offs = ret;
	 
	len = g_list_length(user_prefs->engines);

	for (i=0; i<len; i++)
	{
		src = (gchar*)g_list_nth(user_prefs->engines, i)->data;
		src_len = strlen(src);
//		printf ("toggle engine found: \"%s\"\n", src);
		memcpy (offs, src, src_len);
		offs += src_len;
		if (i<len-1)
		{
			*offs = ';';
			offs++;
		}
	}
	*offs = '\0';
	return ret;
}


void save_prefs()
{
	gchar *togglestring;
	gint width, height;
	
	/* temp cleanup code 
	gnome_config_clean_file (APPLET_WIDGET(window)->privcfgpath);
	gnome_config_sync();
	*/
	
	gnome_config_push_prefix (APPLET_WIDGET(window)->privcfgpath);

	togglestring = build_string_from_toggles();
	//printf ("got toggles string: \"%s\"\n", togglestring);
	
	gnome_config_set_string ("ferret_applet/toggles", togglestring);
	
	if (user_prefs == NULL)
	{
		width = get_int_property ("width", DEFAULT_WIDTH);
		height = get_int_property ("height", DEFAULT_HEIGHT);
	}
	else
	{
		width = user_prefs->width;
		height = user_prefs->height;
	}
	
	gnome_config_set_int ("ferret_applet/width", width);
	gnome_config_set_int ("ferret_applet/height", height);
	
	gnome_config_pop_prefix();		

	gnome_config_sync();
	
//	load_prefs ();
}

/* callback for gnome save session */
gint save_session (GtkWidget *w, gchar *privcfgpath,
				   gchar *globcfgpath, gpointer data)
{
	save_prefs();
	return FALSE;
}


gint prefs_contain_engine (gchar *name, GList *lst)
{
	gint len;
	gint i;
	gchar *s;
	
	if (lst == NULL) return FALSE;
	
	len = g_list_length(lst);
	
	for (i=0; i<len; i++)
	{
		s = (gchar*)g_list_nth(lst, i)->data;
		//printf ("comparing \"%s\" to \"%s\"\n", name, s); 
		if (strcmp(name, s) == 0) return TRUE;
	}
	return FALSE;
}


gint user_prefs_contain_engine (gchar *name)
{
	if (user_prefs == NULL) return FALSE;
	return prefs_contain_engine (name, user_prefs->engines);
}



/*	engines format:
		engine_name;engine_name;...
	any engine name in the toggles list denotes an
	engine which should be displayed.
*/
GList *build_toggles_from_string(gchar *toggles)
{
	gchar *tok;
	gchar *list_entry;
	GList *ret = NULL;
	Tokenizer *st;
	
	//printf ("build_toggles_from_string: %s\n", toggles);
	
	if (strcmp (toggles, "<empty>") == 0)
		return get_default_engine_list();
	
	st = new_tokenizer(toggles, ";");
	
	while (has_more_tokens(st))
	{
		tok = next_token(st);
		//printf ("found token: \"%s\"\n", tok);
		if ((!prefs_contain_engine(tok, ret)) &&
			(engine_for_list_name (tok) != NULL))
		{
#ifdef HDEBUG
			list_entry = h_alloc (strlen(tok)+1, "list_entry", "build_toggles_from_string", 184);
#else
			list_entry = g_malloc (strlen(tok)+1);
#endif
			strcpy (list_entry, tok);
			ret = g_list_append (ret, list_entry);
		}
#ifdef HDEBUG
		h_free (tok, "tok", "build_toggles_from_string", 187);
#else
		g_free (tok);
#endif
	}
	
	free_tokenizer (st);
	return ret;
}

GList *get_preferred_engine_list()
{
	gint len;
	gint i;
	GList *ret = NULL;
	gchar *eng, *engcpy;
	
	/* duplicate the user_prefs->engines
	   list for passage into GUI land */
	
	if ((user_prefs == NULL) || (user_prefs->engines == NULL))
	{
		return get_default_engine_list();
	}
	
	len = g_list_length(user_prefs->engines);
	for (i=0; i<len; i++)
	{
		eng = (gchar*)g_list_nth(user_prefs->engines, i)->data;
#ifdef HDEBUG
		engcpy = h_alloc (strlen(eng)+1, "engcpy", "get_preferred_engine_list", 214);
#else
		engcpy = g_malloc (strlen(eng)+1);
#endif
		strcpy (engcpy, eng);
		//printf ("appending preferred engine: \"%s\"\n", engcpy);
		ret = g_list_append (ret, engcpy);
	}
	return ret;
}

gint load_prefs()
{
	gchar *toggles;
//	gboolean isdefault;
	
	//printf ("attempting to load prefs\n");	
	if (user_prefs != NULL) dispose_prefs();
		
//	printf ("alloc user prefs\n");
#ifdef HDEBUG
	user_prefs = h_alloc (sizeof(Prefs), "user_prefs", "load_prefs", 231);
#else
	user_prefs = g_malloc (sizeof(Prefs));
#endif

	if (user_prefs == NULL) return FALSE;
	
//	printf ("push gnome prefix\n");
	gnome_config_push_prefix (APPLET_WIDGET(window)->privcfgpath);
	//printf ("get \"toggles\" string\n");
	//toggles = gnome_config_get_string_with_default ("ferret_applet/toggles=<empty>", &isdefault);
	toggles = gnome_config_get_string ("ferret_applet/toggles");
	if (toggles == NULL) return FALSE;
	
//	printf ("build user_prefs->engines\n");
	user_prefs->engines = build_toggles_from_string(toggles);
	if (user_prefs->engines == NULL) return FALSE;

#ifdef HDEBUG
	h_free(toggles, "toggles", "load_prefs", 244);
#else
	g_free(toggles);
#endif
	
	user_prefs->width = gnome_config_get_int ("ferret_applet/width");
	if (user_prefs->width <= 0)
		user_prefs->width = get_int_property ("width", DEFAULT_WIDTH);
		
	user_prefs->height = gnome_config_get_int ("ferret_applet/height");
	if (user_prefs->height <= 0)
		user_prefs->height = get_int_property ("height", DEFAULT_HEIGHT);
		
	gnome_config_pop_prefix();

	set_ferret_size (user_prefs->width, user_prefs->height);
	
	return TRUE;
}



void _use_engine (gchar *name)
{
	//printf ("use engine: %s\n", name);
	if (user_prefs == NULL) return;
	if (user_prefs_contain_engine(name)) return;
	user_prefs->engines = g_list_append (user_prefs->engines, name);
}


void _dont_use_engine (gchar *name)
{
	gint len, i;
	GList *leng;
	
	//printf ("\tdon't use engine: %s\n", name);

	if (user_prefs == NULL) return;
	if (!user_prefs_contain_engine(name)) return;
	
	//printf ("\tengine found to be active, disengaging: %s\n", name);
	
	len = g_list_length(user_prefs->engines);
	for (i=0; i<len; i++)
	{
		leng = g_list_nth(user_prefs->engines, i);
		if (strcmp (name, (gchar*)leng->data) == 0)
		{
			user_prefs->engines = g_list_remove_link(user_prefs->engines, leng);
			//printf ("\tlink removed: %s\n", name);
			break;
		}
	}
}



void set_use_engine (gchar *name, gint use)
{
	gchar *dup;
	
	dup = malloc (strlen (name)+1);
	strcpy (dup, name);

	if (use)
		_use_engine(dup);
	else
		_dont_use_engine(dup);
}


void create_default_preferences()
{
#ifdef HDEBUG
	user_prefs = h_alloc(sizeof(Prefs), "user_prefs", "create_default_preferences", 310);
#else
	user_prefs = g_malloc(sizeof(Prefs));
#endif

	user_prefs->engines = get_default_engine_list();
	user_prefs->width = DEFAULT_WIDTH;
	user_prefs->width = DEFAULT_HEIGHT;
}




void dispose_prefs()
{
	gchar *s;
	gint len, i;
	
//	printf ("dispose user prefs\n");

	if (user_prefs == NULL) return;

	len = g_list_length(user_prefs->engines);
	for (i=len-1; i>=0; i--)
	{
		s = (gchar*)g_list_nth(user_prefs->engines, i)->data;
#ifdef HDEBUG
		h_free (s, "s", "dispose_prefs", 365);
#else
		g_free(s);
#endif
	}

	g_list_free (user_prefs->engines);

#ifdef HDEBUG
	h_free (user_prefs, "user_prefs", "dispose_prefs", 373);
#else
	g_free (user_prefs);
#endif

	user_prefs = NULL;
}

