#include "properties.h"
#include "urlencoder.h"


gchar *dont_escape = NULL;
gint dont_escape_len;



/* determine if a given character shoule be escaped. */
gint is_escape_char(gchar c)
{
	gint i;
	
	for (i=0; i<dont_escape_len; i++)
	{
		if (dont_escape[i] == c) return FALSE;
	}
	return TRUE;
}


/*	Escape the "src" char and insert the resulting string
	into "dest". Return the length of the inserted string. */
void insert_escaped_char (gchar *dest, gchar src)
{
	gint len;
	gchar *tmp;
	
	/* all escaped chars take up exactly 3 bytes. */
#ifdef HDEBUG
	tmp = h_alloc (3, "tmp", "insert_escaped_char", 31);
#else
	tmp = g_malloc (3);
#endif	
	/* print src as a hex # into src. */
	sprintf (tmp, "%x", src);
	
	/* find out the length of the returned value */
	len = strlen(tmp);
	
	/* set the first char at dest to the encoding delimiter: '%' */
	*dest = '%';

	/* copy the contents of the tmp string into the dest ptr,
	   and make sure it uses exactly 3 characters. */
	if (len == 2)
		memcpy (dest+1, tmp, len);
	else if (len == 1)
	{
		dest[1]='0';
		memcpy (dest+2, tmp, len);
	}
	
#ifdef HDEBUG
	h_free (tmp, "tmp", "insert_escaped_char", 52);
#else
	g_free (tmp);
#endif

}


/* Escape text for inclusion in an URL */
gchar *escape (gchar *text)
{
	gchar *ret;
	gchar *trim;
	gint oi = 0;
	gint ni = 0;
	gint len = strlen (text);
	gint esclen;
	
	/* Init dont_escape, if we haven't alredy done so */
	if (dont_escape == NULL)
	{
		dont_escape = get_property("dont_escape");
		dont_escape_len = strlen (dont_escape);
	}
	
	/* allocate enough space for three times our strlen
	   to handle the extreme case where all chars
	   are replaced with three character escape codes, with
	   room for a terminating '0' character. Hopefully
	   this is enough for all possible escape sequences... */
	   
#ifdef HDEBUG
	ret = h_alloc ((len*3)+1, "ret", "escape", 78);
#else
	ret = g_malloc ((len*3)+1);
#endif

	/* first, the hard part - convert all
	   non-alphanumeric values to escape codes. */
	
	/* for each char in orig text... */
	for (oi=0; oi<len; oi++)
	{
		/* should we escape this char? */
		if (is_escape_char(text[oi]))
		{
			/* escape, then move the offset forward by 3 */
			insert_escaped_char (ret + ni, text[oi]);
			ni += 3;
		}
		else
		{
			/* otherwise just copy a single char */
			ret[ni++] = text[oi];
		}
	}
	
	/* Make "ret" a bone-fide c string - mark the
	   end with a '0'; then remember its' length */
	ret[ni] = 0;
	esclen = ni;
	
	/* next, the easy part... convert space chars to
	   plus symbols */
	for (ni=0; ni<esclen; ni++)
		if ((ret[ni]==' ') || (ret[ni]=='\t'))
			ret[ni] = '+';
	
	/* Return a string which is sized more accurately than ret */
#ifdef HDEBUG
	trim = h_alloc (esclen+1, "trim", "escape", 112);
#else
	trim = g_malloc (esclen+1);
#endif	
	memcpy (trim, ret, esclen+1);
	
//	printf ("orig string:    \"%s\"\n", text);
//	printf ("encoded string: \"%s\"\n", trim);

#ifdef HDEBUG
	h_free (ret, "ret", "escape", 118);
#else
	g_free (ret);
#endif	
	
	return trim;
}


void reset_urlencoder_properties()
{
	dont_escape = NULL;
}
